@@ -1,15 +1,6 @@
-import sys
-
 from minfraud.models import *
 
-if sys.version_info[:2] == (2, 6):
-    import unittest2 as unittest
-else:
-    import unittest
-
-if sys.version_info[0] == 2:
-    unittest.TestCase.assertRaisesRegex = unittest.TestCase.assertRaisesRegexp
-    unittest.TestCase.assertRegex = unittest.TestCase.assertRegexpMatches
+import unittest
 
 
 class TestModels(unittest.TestCase):
@@ -32,9 +23,9 @@ class TestModels(unittest.TestCase):
         for model in models:
             for attr in (model.attr, "does_not_exist"):
                 with self.assertRaises(
-                    AttributeError, msg="{0!s} - {0}".format(model.obj, attr)
+                    AttributeError, msg="{0!s} - {1}".format(model.obj, attr)
                 ):
-                    setattr(model.obj, attr, 5)
+                    setattr(model.obj, attr, 5)  # type: ignore
 
     def test_billing_address(self):
         address = BillingAddress(self.address_dict)
@@ -265,13 +256,16 @@ class TestModels(unittest.TestCase):
         self.assertEqual(0.06, factors.subscores.country)
         self.assertEqual(0.07, factors.subscores.country_mismatch)
         self.assertEqual(0.08, factors.subscores.cvv_result)
+        self.assertEqual(0.18, factors.subscores.device)
         self.assertEqual(0.09, factors.subscores.email_address)
         self.assertEqual(0.10, factors.subscores.email_domain)
+        self.assertEqual(0.19, factors.subscores.email_local_part)
         self.assertEqual(0.11, factors.subscores.email_tenure)
         self.assertEqual(0.12, factors.subscores.ip_tenure)
         self.assertEqual(0.13, factors.subscores.issuer_id_number)
         self.assertEqual(0.14, factors.subscores.order_amount)
         self.assertEqual(0.15, factors.subscores.phone_number)
+        self.assertEqual(0.2, factors.subscores.shipping_address)
         self.assertEqual(
             0.16, factors.subscores.shipping_address_distance_to_ip_location
         )
@@ -304,13 +298,16 @@ class TestModels(unittest.TestCase):
                 "country": 0.06,
                 "country_mismatch": 0.07,
                 "cvv_result": 0.08,
+                "device": 0.18,
                 "email_address": 0.09,
                 "email_domain": 0.10,
+                "email_local_part": 0.19,
                 "email_tenure": 0.11,
                 "ip_tenure": 0.12,
                 "issuer_id_number": 0.13,
                 "order_amount": 0.14,
                 "phone_number": 0.15,
+                "shipping_address": 0.2,
                 "shipping_address_distance_to_ip_location": 0.16,
                 "time_of_day": 0.17,
             },
